@@ -5,21 +5,41 @@
 # later we can have a separate changelist to refactor main.py into smaller files
 # pylint: disable=too-many-lines
 
+from typing import List, Optional, Union
+from types import ModuleType
+
 import argparse
+argcomplete: Union[None, ModuleType] = None
+try:
+    import argcomplete
+except ImportError as e:
+    pass # already set to None by default above
+
 import logging
 import os
 import platform
 import sys
 import time
-from typing import List, Optional
 
-import pyqrcode  # type: ignore[import-untyped]
+try:
+    import pyqrcode  # type: ignore[import-untyped]
+except ImportError as e:
+    pyqrcode = None
+
 import yaml
 from google.protobuf.json_format import MessageToDict
 from pubsub import pub  # type: ignore[import-untyped]
 
-import meshtastic.test
+try:
+    import meshtastic.test
+    have_test = True
+except ImportError as e:
+    have_test = False
+
 import meshtastic.util
+import meshtastic.serial_interface
+import meshtastic.tcp_interface
+
 from meshtastic import BROADCAST_ADDR, mt_config, remote_hardware
 from meshtastic.ble_interface import BLEInterface
 from meshtastic.mesh_interface import MeshInterface
@@ -54,7 +74,7 @@ def onReceive(packet, interface) -> None:
             args
             and args.sendtext
             and packet["to"] == interface.myInfo.my_node_num
-            and d["portnum"] == portnums_pb2.PortNum.TEXT_MESSAGE_APP
+            and d.get("portnum", portnums_pb2.PortNum.UNKNOWN_APP) == portnums_pb2.PortNum.TEXT_MESSAGE_APP
         ):
             interface.close()  # after running command then exit
 
@@ -70,7 +90,7 @@ def onReceive(packet, interface) -> None:
                 interface.sendText(reply)
 
     except Exception as ex:
-        print(f"Warning: There is no field {ex} in the packet.")
+        print(f"Warning: Error processing received packet: {ex}.")
 
 
 def onConnection(interface, topic=pub.AUTO_TOPIC) -> None:  # pylint: disable=W0613
@@ -432,6 +452,26 @@ def onConnected(interface):
             waitForAckNak = True
             interface.getNode(args.dest, False, **getNode_kwargs).removeNode(args.remove_node)
 
+        if args.set_favorite_node:
+            closeNow = True
+            waitForAckNak = True
+            interface.getNode(args.dest, False, **getNode_kwargs).setFavorite(args.set_favorite_node)
+
+        if args.remove_favorite_node:
+            closeNow = True
+            waitForAckNak = True
+            interface.getNode(args.dest, False, **getNode_kwargs).removeFavorite(args.remove_favorite_node)
+
+        if args.set_ignored_node:
+            closeNow = True
+            waitForAckNak = True
+            interface.getNode(args.dest, False, **getNode_kwargs).setIgnored(args.set_ignored_node)
+
+        if args.remove_ignored_node:
+            closeNow = True
+            waitForAckNak = True
+            interface.getNode(args.dest, False, **getNode_kwargs).removeIgnored(args.remove_ignored_node)
+
         if args.reset_nodedb:
             closeNow = True
             waitForAckNak = True
@@ -891,8 +931,11 @@ def onConnected(interface):
             else:
                 urldesc = "Primary channel URL"
             print(f"{urldesc}: {url}")
-            qr = pyqrcode.create(url)
-            print(qr.terminal())
+            if pyqrcode is not None:
+                qr = pyqrcode.create(url)
+                print(qr.terminal())
+            else:
+                print("Install pyqrcode to view a QR code printed to terminal.")
 
         log_set: Optional = None  # type: ignore[annotation-unchecked]
         # we need to keep a reference to the logset so it doesn't get GCed early
@@ -1143,11 +1186,14 @@ def common():
             parser.print_help(sys.stderr)
             meshtastic.util.our_exit("", 1)
         elif args.test:
-            result = meshtastic.test.testAll()
-            if not result:
-                meshtastic.util.our_exit("Warning: Test was not successful.")
+            if not have_test:
+                meshtastic.util.our_exit("Test module could not be important. Ensure you have the 'dotmap' module installed.")
             else:
-                meshtastic.util.our_exit("Test was a success.", 0)
+                result = meshtastic.test.testAll()
+                if not result:
+                    meshtastic.util.our_exit("Warning: Test was not successful.")
+                else:
+                    meshtastic.util.our_exit("Test was a success.", 0)
         else:
             if args.seriallog == "stdout":
                 logfile = sys.stdout
@@ -1684,6 +1730,26 @@ def addRemoteAdminArgs(parser: argparse.ArgumentParser) -> argparse.ArgumentPars
         help="Tell the destination node to remove a specific node from its DB, by node number or ID",
         metavar="!xxxxxxxx"
     )
+    group.add_argument(
+        "--set-favorite-node",
+        help="Tell the destination node to set the specified node to be favorited on the NodeDB on the devicein its DB, by number or ID",
+        metavar="!xxxxxxxx"
+    )
+    group.add_argument(
+        "--remove-favorite-node",
+        help="Tell the destination node to set the specified node to be un-favorited on the NodeDB on the device, by number or ID",
+        metavar="!xxxxxxxx"
+    )
+    group.add_argument(
+        "--set-ignored-node",
+        help="Tell the destination node to set the specified node to be ignored on the NodeDB on the devicein its DB, by number or ID",
+        metavar="!xxxxxxxx"
+    )
+    group.add_argument(
+        "--remove-ignored-node",
+        help="Tell the destination node to set the specified node to be un-ignored on the NodeDB on the device, by number or ID",
+        metavar="!xxxxxxxx"
+    )
     group.add_argument(
         "--reset-nodedb",
         help="Tell the destination node to clear its list of nodes",
@@ -1903,6 +1969,8 @@ def initParser():
 
     parser.set_defaults(deprecated=None)
 
+    if argcomplete is not None:
+        argcomplete.autocomplete(parser)
     args = parser.parse_args()
     mt_config.args = args
     mt_config.parser = parser
