@@ -5,7 +5,9 @@
 import collections
 import json
 import logging
+import math
 import random
+import secrets
 import sys
 import threading
 import time
@@ -15,7 +17,11 @@ from decimal import Decimal
 from typing import Any, Callable, Dict, List, Optional, Union
 
 import google.protobuf.json_format
-import print_color  # type: ignore[import-untyped]
+try:
+    import print_color  # type: ignore[import-untyped]
+except ImportError as e:
+    print_color = None
+
 from pubsub import pub  # type: ignore[import-untyped]
 from tabulate import tabulate
 
@@ -153,7 +159,7 @@ class MeshInterface:  # pylint: disable=R0902
     @staticmethod
     def _printLogLine(line, interface):
         """Print a line of log output."""
-        if interface.debugOut == sys.stdout:
+        if print_color is not None and interface.debugOut == sys.stdout:
             # this isn't quite correct (could cause false positives), but currently our formatting differs between different log representations
             if "DEBUG" in line:
                 print_color.print(line, color="cyan", end=None)
@@ -263,6 +269,7 @@ class MeshInterface:  # pylint: disable=R0902
                             "ID": user["id"],
                             "Hardware": user.get("hwModel", "UNSET"),
                             "Pubkey": user.get("publicKey", "UNSET"),
+                            "Role": user.get("role", "N/A"),
                         }
                     )
 
@@ -299,7 +306,7 @@ class MeshInterface:  # pylint: disable=R0902
                 row.update(
                     {
                         "SNR": formatFloat(node.get("snr"), 2, " dB"),
-                        "Hops Away": node.get("hopsAway", "0/unknown"),
+                        "Hops": node.get("hopsAway", "?"),
                         "Channel": node.get("channel", 0),
                         "LastHeard": getLH(node.get("lastHeard")),
                         "Since": getTimeAgo(node.get("lastHeard")),
@@ -385,6 +392,40 @@ class MeshInterface:  # pylint: disable=R0902
             channelIndex=channelIndex,
         )
 
+
+    def sendAlert(
+        self,
+        text: str,
+        destinationId: Union[int, str] = BROADCAST_ADDR,
+        onResponse: Optional[Callable[[dict], Any]] = None,
+        channelIndex: int = 0,
+    ):
+        """Send an alert text to some other node. This is similar to a text message, 
+            but carries a higher priority and is capable of generating special notifications
+            on certain clients.
+
+        Arguments:
+            text {string} -- The text of the alert to send
+
+        Keyword Arguments:
+            destinationId {nodeId or nodeNum} -- where to send this
+                                                 message (default: {BROADCAST_ADDR})
+
+        Returns the sent packet. The id field will be populated in this packet
+        and can be used to track future message acks/naks.
+        """
+
+        return self.sendData(
+            text.encode("utf-8"),
+            destinationId,
+            portNum=portnums_pb2.PortNum.ALERT_APP,
+            wantAck=False,
+            wantResponse=False,
+            onResponse=onResponse,
+            channelIndex=channelIndex,
+            priority=mesh_pb2.MeshPacket.Priority.ALERT
+        )
+
     def sendData(
         self,
         data,
@@ -398,6 +439,7 @@ class MeshInterface:  # pylint: disable=R0902
         hopLimit: Optional[int]=None,
         pkiEncrypted: Optional[bool]=False,
         publicKey: Optional[bytes]=None,
+        priority: mesh_pb2.MeshPacket.Priority.ValueType=mesh_pb2.MeshPacket.Priority.RELIABLE,
     ): # pylint: disable=R0913
         """Send a data packet to some other node
 
@@ -449,6 +491,8 @@ class MeshInterface:  # pylint: disable=R0902
         meshPacket.decoded.portnum = portNum
         meshPacket.decoded.want_response = wantResponse
         meshPacket.id = self._generatePacketId()
+        if priority is not None:
+            meshPacket.priority = priority
 
         if onResponse is not None:
             logging.debug(f"Setting a response handler for requestId {meshPacket.id}")
@@ -696,6 +740,113 @@ class MeshInterface:  # pylint: disable=R0902
                     "No response from node. At least firmware 2.1.22 is required on the destination node."
                 )
 
+    def onResponseWaypoint(self, p: dict):
+        """on response for waypoint"""
+        if p["decoded"]["portnum"] == "WAYPOINT_APP":
+            self._acknowledgment.receivedWaypoint = True
+            w = mesh_pb2.Waypoint()
+            w.ParseFromString(p["decoded"]["payload"])
+            print(f"Waypoint received: {w}")
+        elif p["decoded"]["portnum"] == "ROUTING_APP":
+            if p["decoded"]["routing"]["errorReason"] == "NO_RESPONSE":
+                our_exit(
+                    "No response from node. At least firmware 2.1.22 is required on the destination node."
+                )
+
+    def sendWaypoint(
+        self,
+        name,
+        description,
+        expire: int,
+        waypoint_id: Optional[int] = None,
+        latitude: float = 0.0,
+        longitude: float = 0.0,
+        destinationId: Union[int, str] = BROADCAST_ADDR,
+        wantAck: bool = True,
+        wantResponse: bool = False,
+        channelIndex: int = 0,
+    ): # pylint: disable=R0913
+        """
+        Send a waypoint packet to some other node (normally a broadcast)
+
+        Returns the sent packet. The id field will be populated in this packet and
+        can be used to track future message acks/naks.
+        """
+        w = mesh_pb2.Waypoint()
+        w.name = name
+        w.description = description
+        w.expire = expire
+        if waypoint_id is None:
+            # Generate a waypoint's id, NOT a packet ID.
+            # same algorithm as https://github.com/meshtastic/js/blob/715e35d2374276a43ffa93c628e3710875d43907/src/meshDevice.ts#L791
+            seed = secrets.randbits(32)
+            w.id = math.floor(seed * math.pow(2, -32) * 1e9)
+            logging.debug(f"w.id:{w.id}")
+        else:
+            w.id = waypoint_id
+        if latitude != 0.0:
+            w.latitude_i = int(latitude * 1e7)
+            logging.debug(f"w.latitude_i:{w.latitude_i}")
+        if longitude != 0.0:
+            w.longitude_i = int(longitude * 1e7)
+            logging.debug(f"w.longitude_i:{w.longitude_i}")
+
+        if wantResponse:
+            onResponse = self.onResponseWaypoint
+        else:
+            onResponse = None
+
+        d = self.sendData(
+            w,
+            destinationId,
+            portNum=portnums_pb2.PortNum.WAYPOINT_APP,
+            wantAck=wantAck,
+            wantResponse=wantResponse,
+            onResponse=onResponse,
+            channelIndex=channelIndex,
+        )
+        if wantResponse:
+            self.waitForWaypoint()
+        return d
+
+    def deleteWaypoint(
+        self,
+        waypoint_id: int,
+        destinationId: Union[int, str] = BROADCAST_ADDR,
+        wantAck: bool = True,
+        wantResponse: bool = False,
+        channelIndex: int = 0,
+    ):
+        """
+        Send a waypoint deletion packet to some other node (normally a broadcast)
+
+        NB: The id must be the waypoint's id and not the id of the packet creation.
+        
+        Returns the sent packet. The id field will be populated in this packet and
+        can be used to track future message acks/naks.
+        """
+        p = mesh_pb2.Waypoint()
+        p.id = waypoint_id
+        p.expire = 0
+
+        if wantResponse:
+            onResponse = self.onResponseWaypoint
+        else:
+            onResponse = None
+
+        d = self.sendData(
+            p,
+            destinationId,
+            portNum=portnums_pb2.PortNum.WAYPOINT_APP,
+            wantAck=wantAck,
+            wantResponse=wantResponse,
+            onResponse=onResponse,
+            channelIndex=channelIndex,
+        )
+        if wantResponse:
+            self.waitForWaypoint()
+        return d
+
     def _addResponseHandler(
         self,
         requestId: int,
@@ -820,6 +971,12 @@ class MeshInterface:  # pylint: disable=R0902
         if not success:
             raise MeshInterface.MeshInterfaceError("Timed out waiting for position")
 
+    def waitForWaypoint(self):
+        """Wait for waypoint"""
+        success = self._timeout.waitForWaypoint(self._acknowledgment)
+        if not success:
+            raise MeshInterface.MeshInterfaceError("Timed out waiting for waypoint")
+
     def getMyNodeInfo(self) -> Optional[Dict]:
         """Get info about my node."""
         if self.myInfo is None or self.nodesByNum is None:
