@@ -1608,7 +1608,7 @@ def test_main_onReceive_empty(caplog, capsys):
     assert re.search(r"in onReceive", caplog.text, re.MULTILINE)
     out, err = capsys.readouterr()
     assert re.search(
-        r"Warning: There is no field 'to' in the packet.", out, re.MULTILINE
+        r"Warning: Error processing received packet: 'to'.", out, re.MULTILINE
     )
     assert err == ""
 
@@ -2652,3 +2652,64 @@ def test_tunnel_tunnel_arg(
         out, err = capsys.readouterr()
         assert re.search(r"Connected to radio", out, re.MULTILINE)
         assert err == ""
+
+
+@pytest.mark.unit
+@pytest.mark.usefixtures("reset_mt_config")
+def test_set_favorite_node():
+    """Test --set-favorite-node node"""
+    sys.argv = ["", "--set-favorite-node", "!12345678"]
+    mt_config.args = sys.argv
+    mocked_node = MagicMock(autospec=Node)
+    iface = MagicMock(autospec=SerialInterface)
+    iface.getNode.return_value = mocked_node
+    with patch("meshtastic.serial_interface.SerialInterface", return_value=iface):
+        main()
+
+    mocked_node.setFavorite.assert_called_once_with("!12345678")
+
+
+@pytest.mark.unit
+@pytest.mark.usefixtures("reset_mt_config")
+def test_remove_favorite_node():
+    """Test --remove-favorite-node node"""
+    sys.argv = ["", "--remove-favorite-node", "!12345678"]
+    mt_config.args = sys.argv
+    mocked_node = MagicMock(autospec=Node)
+    iface = MagicMock(autospec=SerialInterface)
+    iface.getNode.return_value = mocked_node
+    mocked_node.iface = iface
+    with patch("meshtastic.serial_interface.SerialInterface", return_value=iface):
+        main()
+
+    mocked_node.removeFavorite.assert_called_once_with("!12345678")
+
+@pytest.mark.unit
+@pytest.mark.usefixtures("reset_mt_config")
+def test_set_ignored_node():
+    """Test --set-ignored-node node"""
+    sys.argv = ["", "--set-ignored-node", "!12345678"]
+    mt_config.args = sys.argv
+    mocked_node = MagicMock(autospec=Node)
+    iface = MagicMock(autospec=SerialInterface)
+    iface.getNode.return_value = mocked_node
+    with patch("meshtastic.serial_interface.SerialInterface", return_value=iface):
+        main()
+
+    mocked_node.setIgnored.assert_called_once_with("!12345678")
+
+
+@pytest.mark.unit
+@pytest.mark.usefixtures("reset_mt_config")
+def test_remove_ignored_node():
+    """Test --remove-ignored-node node"""
+    sys.argv = ["", "--remove-ignored-node", "!12345678"]
+    mt_config.args = sys.argv
+    mocked_node = MagicMock(autospec=Node)
+    iface = MagicMock(autospec=SerialInterface)
+    iface.getNode.return_value = mocked_node
+    mocked_node.iface = iface
+    with patch("meshtastic.serial_interface.SerialInterface", return_value=iface):
+        main()
+
+    mocked_node.removeIgnored.assert_called_once_with("!12345678")
